package org.rcisoft.business.opmArticle.service.impl;


import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.baidu.censor.dto.CensorResult;
import org.rcisoft.baidu.censor.service.ContentCensorService;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.opmArticle.dao.OpmArticleRepository;
import org.rcisoft.business.opmArticle.entity.ArticleCommentDTO;
import org.rcisoft.business.opmArticle.entity.ArticleCommentVO;
import org.rcisoft.business.opmArticle.entity.LikeDTO;
import org.rcisoft.business.opmArticle.entity.OpmArticle;
import org.rcisoft.business.opmArticle.service.OpmArticleService;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.service.CyFileStorageService;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.dictionary.entity.DictData;
import org.rcisoft.sys.dictionary.service.DictionaryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by cy on 2024年3月30日 下午1:51:15.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class OpmArticleServiceImpl extends ServiceImpl<OpmArticleRepository, OpmArticle> implements OpmArticleService {

    @Autowired
    private DictionaryService dictionaryService;
    @Autowired
    private ContentCensorService contentCensorService;
    @Autowired
    private CyFileStorageService cyFileStorageService;
    @Autowired
    private MemInfoRepository memInfoRepository;
    /**
    * 保存 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(OpmArticle opmArticle){
        //从redis获取动态审核配置  0：自动审核，1：人工审核
        List<DictData> articleAudit = dictionaryService.selectByTypes("article_audit_config");
        String style = articleAudit.get(0).getDictValue();
        if ("0".equals(style)) {
            //调用百度，自动审核
            boolean flag = false;
            CensorResult result = contentCensorService.getCommonTextCensorResult(opmArticle.getContent());
            flag = result.getIsPass();
            List<String> msg = new ArrayList<>();
            if (!flag)
                msg.add(result.getErrorMsg());
            if (StringUtils.isNotBlank(opmArticle.getPictureId())){
                JSONArray ids = JSONUtil.parseObj(opmArticle.getPictureId()).getJSONArray("id");
                for (Object id : ids) {
                    String url = cyFileStorageService.getOssUrl((Integer)id);
                    CensorResult imgResult = contentCensorService.getImageCensorResult(url);
                    if (!imgResult.getIsPass())
                        msg.add(result.getErrorMsg());
                    flag = flag && imgResult.getIsPass();
                }
            }
            if (flag)
                opmArticle.setExamStatus("1");
            else {
                opmArticle.setExamStatus("3");
                opmArticle.setRemarks(String.join("，", msg));
            }
        }
        //增加操作
        int line = baseMapper.insert(opmArticle);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
                opmArticle.getBusinessId()+"的opmArticle管理信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(OpmArticle opmArticle){
        int line = baseMapper.realDelete(opmArticle);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
                opmArticle.getBusinessId()+"的opmArticle管理信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(OpmArticle opmArticle){
        opmArticle.setDeleted();
        int line = baseMapper.deleteById(opmArticle);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                    opmArticle.getBusinessId()+"的opmArticle管理信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(OpmArticle opmArticle){
        int line = baseMapper.updateById(opmArticle);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ opmArticle.getBusinessId()+"的opmArticle管理信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 opmArticle管理
    * @param articleId
    * @return
    */
    @Override
    public OpmArticle findById(Integer articleId){
        String loginUserId = CyUserUtil.getAuthenBusinessId();
        //根据动态id 查询动态详情
        OpmArticle opmArticle = baseMapper.selectArticleDetail(articleId,loginUserId);
        //查询该动态的评论
        List<ArticleCommentVO> list = baseMapper.selectArticleComment(opmArticle.getBusinessId(),loginUserId);
        list.forEach(item -> {
            //查询该条评论下的子评论
            List<ArticleCommentVO> childrenCommentList = baseMapper.selectArticleChildrenComment(opmArticle.getBusinessId(),item.getBusinessId(),loginUserId);
            item.setChildrenCommentList(childrenCommentList);
        });
        opmArticle.setArticleCommentVOList(list);
        return opmArticle;
    }

    /**
    * 分页查询 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Override
    public IPage<OpmArticle> findAllByPagination(CyPageInfo<OpmArticle> paginationUtility,
                                                 OpmArticle opmArticle){
        IPage<OpmArticle> result = null;
        if (opmArticle.getArticleType() != null && opmArticle.getArticleType().equals("0")){
            //关注
            //查询该用户关注对象的id集合
            List<Integer> list = memInfoRepository.queryFollowIdById(CyUserUtil.getAuthenBusinessId());
            opmArticle.setFollowList(list);
            if (list.isEmpty()){
                result = paginationUtility;
            } else {
                result = baseMapper.queryOpmArticlePaged(paginationUtility,opmArticle);
            }
        } else {
            if (StringUtils.isNotEmpty(opmArticle.getUserId())){
                //设置查询用户 查询某个人的动态列表
                opmArticle.setUserId(opmArticle.getUserId());
            }
            if (StringUtils.isNotEmpty(CyUserUtil.getAuthenBusinessId())){
                //设置当前登录人id 用来查询该用户对文章是否已点赞
                opmArticle.setLoginUserId(CyUserUtil.getAuthenBusinessId());
            }
            result = baseMapper.queryOpmArticlePaged(paginationUtility,opmArticle);
        }
        return result;

    }

    /**
    * 查询list opmArticle管理
    * @param opmArticle
    * @return
    */
    @Override
    public List<OpmArticle> findAll(OpmArticle opmArticle){
        return baseMapper.queryOpmArticle(opmArticle);

    }

    /**
    * 导出opmArticle管理
    * @return
    */
    @Override
    public List<OpmArticle> export(OpmArticle opmArticle) {
        List<OpmArticle> opmArticleList = baseMapper.queryOpmArticle(opmArticle);
        return opmArticleList;
    }


    /**
     * 逻辑删除 评论
     * @param opmArticle
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeComment(OpmArticle opmArticle){
        opmArticle.setDeleted();
        int line = baseMapper.deleteComment(opmArticle);
        log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                opmArticle.getBusinessId()+"的opmArticleComment信息");
        return new CyPersistModel(line);
    }

    /**
     * 点赞增加点赞数
     * @param articleId
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel likeAdd(Integer articleId,String praiseType){
        int line = 0;
        if (praiseType.equals("praise")){
            //点赞
            //向点赞表增加一条数据
            LikeDTO likeDTO = new LikeDTO();
            likeDTO.setUserId(CyUserUtil.getAuthenBusinessId());
            likeDTO.setArticleId(articleId);
            //增加主表该动态的点赞数
            baseMapper.addLikeCount(articleId);
            line = baseMapper.likeAdd(likeDTO);
        } else if (praiseType.equals("cancel")){
            //取消点赞
            //删除点赞表中的数据
            String userId = CyUserUtil.getAuthenBusinessId();
            baseMapper.likeDelete(userId,articleId);
            //修改主表中该动态的点赞数
            line = baseMapper.reduceLikeCount(articleId);
        } else {
            throw new CyServiceException("点赞失败，请稍后再试");
        }
        return new CyPersistModel(line);
    }

    /**
     * 评论增加点赞数
     * @param commentId
     * @param praiseType  点赞类型 praise:点赞  cancel:取消点赞
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel likeCommentAdd(Integer commentId,String praiseType){
        int line = 0;
        if (praiseType.equals("praise")){
            //点赞
            //向点赞表增加一条数据
            LikeDTO likeDTO = new LikeDTO();
            likeDTO.setUserId(CyUserUtil.getAuthenBusinessId());
            likeDTO.setCommentId(commentId);
            //增加主表该动态的点赞数
            baseMapper.addCommentLikeCount(commentId);
            line = baseMapper.likeAdd(likeDTO);
        } else if (praiseType.equals("cancel")){
            //取消点赞
            //删除点赞表中的数据
            String userId = CyUserUtil.getAuthenBusinessId();
            baseMapper.likeDelete(userId,commentId);
            //修改主表中该动态的点赞数
            line = baseMapper.reduceCommentLikeCount(commentId);
        } else {
            throw new CyServiceException("点赞失败，请稍后再试");
        }
        return new CyPersistModel(line);
    }

    /**
     * 添加评论
     * @param dto
     * @return
     */
    @Override
    public CyPersistModel addComment(ArticleCommentDTO dto) {
        dto.setCommentedUserId(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        dto.setCreateDate(new Date());
        dto.setUpdateDate(new Date());
        dto.setCreateBy(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        dto.setUpdateBy(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        int line = baseMapper.addComment(dto);
        //修改主表的评论数
        baseMapper.addArticleCommentCount(dto.getArticleId());
        //修改子表的评论数 给父级评论数+1
        if (dto.getParentId() != null){
            baseMapper.addCommentCount(dto.getParentId());
        }
        return new CyPersistModel(line);
    }
}
